//===========================================================================
//  FLOPTEST.C - check status of floppy drive                 
//  William Zentmayer 06-JAN-92
// -------------------------------------------------------------------------
//  prototype:
//      int floptest(char drive);
//
//  parameters:
//      char drive = floppy drive letter to test A=0,B=1,...
//
//  returns:  
//        0 (0x00) = no error
//        1 (0x01) = invalid command (fixed disk)
//        3 (0x03) = diskette write-protected
//       12 (0x0C) = invalid media (incorrect density)
//      128 (0x80) = drive time-out (disk not present or drive door open)
//      all others = other system errors
// -------------------------------------------------------------------------
//  Other functions:
//      void int13(union REGS *, union REGS *, struct SREGS *);
//===========================================================================
#include <dos.h>
#include <stdio.h>
#include <ctype.h>

#define SECTOR_SIZE 512                          // floppy sector size

void int13(union REGS *, union REGS *, struct SREGS *);

int floptest(char drive)
{
   union REGS InRegs, OutRegs;
   struct SREGS SegRegs;
   char sector_data[SECTOR_SIZE];
   char far *buffer=sector_data;
   int drv=toupper(drive)-'A';

   InRegs.h.al=0x01;                             // number of sectors
   InRegs.h.ch=0x00;                             // cylinder       
   InRegs.h.cl=0x01;                             // sector              
   InRegs.h.dh=0x00;                             // head                 
   InRegs.h.dl=drv;                              // drive number 0=A,1=B,...
   SegRegs.es =FP_SEG(buffer);                   // segment of buffer
   InRegs.x.bx=FP_OFF(buffer);                   // offset of buffer

   InRegs.h.ah=0x02;                             // read sector
   int13(&InRegs,&OutRegs,&SegRegs);             // int 13h func 02h

   if(!OutRegs.h.ah)                             // read successful
       {
       InRegs.h.ah=0x03;                         // write sector
       int13(&InRegs,&OutRegs,&SegRegs);         // int 13h func 03h
       }

   return OutRegs.h.ah;
}

void int13(union REGS *InRegs, union REGS *OutRegs, struct SREGS *SegRegs)
{
   register int i;
   int holdreg=InRegs-> h.ah;

   for(i=0;i<3;i++)
       {
       InRegs-> h.ah=0x00;                       // reset controller
       int86(0x13,InRegs,OutRegs);               // int 13h func 00h
       if(OutRegs-> h.ah)                        // reset failed     
           break;                                // exit loop - error

       InRegs-> h.ah=holdreg;                    // called int 13h function
       int86x(0x13,InRegs,OutRegs,SegRegs);      // interrupt 13h
       if(OutRegs-> h.ah)                        // function failed
           continue;                             // retry
       }
}


         
