/* PCOUNT - A 'C' program written to gather statistics from Pascal
 *   source programs.  The main number to count is non-comment source
 *   statements (NCSS).  We also report the number of PROCEDURE + FUNCTION
 *   statements, comments, blank lines, and total source lines.
 *   NCSS is estimated by counting the number of semicolons, not counting
 *   those in string constants or comments.  Both "{" and "(*" form of
 *   comments are counted. A line that contains a semicolon and a comment
 *   will be counted in both categories.  
 *   The program was originally compiled in Turbo C. If it is linked with
 *   "WILDARGS.OBJ", wildcards can be used for the file name.  If no input
 *   parameter is supplied, it reads the standard input stream. Output is
 *   both to the console and to a file "PCOUNT.LST".
 * Usage:   pcount file1.pas file2.pas ... 
 *          (wildcards are allowed also, e.g. *.pas)
 *
 *   Written by Ted Shapin, 2/9/89.
 */

#include <stdio.h>
#include <ctype.h>

#define normal 0
#define incomment 16
#define inquote 17
#define inparens 18
#define inqinp 19 /* found a quote while inside a parenthesis */

long    tc      = 0;  /* total comments */ 
long    tl      = 0;  /* total lines */ 
long    ts      = 0;  /* total non-comment semicolons */ 
long    tb      = 0;  /* total blank lines */ 
long    tp      = 0;  /* total procedures or functions */
int     keycnt  = 0;  /* scan for "function" or "procedure" */ 
int     state   = 0;  /* tells what we have seen */ 
char    file_name[81]; 
        FILE *fp_out;
 
main(argc, argv)
char *argv[];
{
        register int    i, tf;
        register FILE   *fp;
        int             gotcha;

        tf = 0;
   if( (fp_out = fopen("pcount.lst", "w")) == NULL )
   {
      fprintf( stderr, "can't open %s\n", "pcount.lst" );
      exit(1);
   }

   /* Count each file on the command line,
    * and write the list to the file funcs.txt.
    */
   printf( "%13s:\t  NCSS  Comnts  Procs  Blanks  Lines\n","pcount 1.0");
   fprintf(fp_out, "%13s:\t  NCSS  Comnts  Procs  Blanks  Lines\n",
         "pcount 1.0");
        if(argc < 2) {
                ++tf;
                count(stdin, 0);
        } else
                for (i = 1; i < argc; ++i) {
                        if ((fp = fopen(argv[i], "r")) == NULL) {

                                fprintf(stderr, "\"%s\": cannot open\n",
                                                argv[i]);
                                continue;
                        }
                        ++tf;
                        count(fp, argv[i]);
                        fclose(fp);
                }
      if (tf > 1) {
      printf( "%13s:\t%6ld %6ld %6ld %6ld %6ld \n", "Totals", ts,tc,tp,tb,tl);
      fprintf( fp_out, "%13s:\t%6ld %6ld %6ld %6ld %6ld\n","Totals", 
               ts,tc,tp,tb,tl);}
      fclose(fp_out);
}
 
count(fp, fn)    /* get counts for one file */
char *fn;
FILE *fp;
{
        register int c, lastc;
        long    fc      = 0;  /* file comments */
        long    fl      = 0;  /* file lines */
        long    fs      = 0;  /* file non-comment semicolons */
        long    fb      = 0;  /* file blank lines */
        long    ff      = 0;  /* file procedures or functions */
        int     parenct = 0;  /* parenthesis count */
        int     llastc  = 0;  /* char before last */
        lastc = 0;
        while((c=tolower(getc(fp))) != EOF)
                {
                if(c == '\n') {++fl;
                         if(lastc=='\n') ++fb;}

                switch (state)
                      {
                      case normal:
                         if(c=='\'') {state=inquote; keycnt=0; break;}
                         if(c=='{') {state=incomment; ++fc; keycnt=0; break;}
                         if(c=='(') 
                              {state=inparens; ++parenct; keycnt=0; break;}
                         if(c==';') {++fs; keycnt=0; break;}
                         switch (keycnt)
                          {
                          case 0:
                           if (c=='r' && lastc =='p' && !isalpha(llastc))
                                   {++keycnt;break;}
                           if (c=='u' && lastc =='f'&& !isalpha(llastc)) 
                                   {keycnt=9;break;}
                           break;
                          case 1: {if (c=='o') ++keycnt;
                                 else keycnt=0;} break;
                          case 2: {if (c=='c') ++keycnt;
                                 else keycnt=0;} break;
                          case 3: {if (c=='e') ++keycnt;
                                 else keycnt=0;} break;
                          case 4: {if (c=='d') ++keycnt;
                                 else keycnt=0;} break;
                          case 5: {if (c=='u') ++keycnt;
                                 else keycnt=0;} break;
                          case 6: {if (c=='r') ++keycnt;
                                 else keycnt=0;} break;
                          case 7: {if (c=='e') ++keycnt;
                                 else keycnt=0;} break;
                          case 8:  if (c==' ') {++ff; keycnt=0;}
                                 break;
                          case 9:  {if (c=='n') ++keycnt;
                                   else keycnt=0;} break;
                          case 10: {if (c=='c') ++keycnt;
                                 else keycnt=0;} break;
                          case 11: {if (c=='t') ++keycnt;
                                 else keycnt=0;} break;
                          case 12: {if (c=='i') ++keycnt;
                                 else keycnt=0;} break;
                          case 13: {if (c=='o') ++keycnt;
                                 else keycnt=0;} break;
                          case 14: {if (c=='n') ++keycnt;
                                 else keycnt=0;} break;
                          case 15: {if (c==' ') {++ff; keycnt=0;}}
                                 break;
                          default: state=0; break;
                          }
                          break;
                      case incomment:  if(c=='}') {state=normal;break;}
                                       if(c==')'&& lastc=='*') 
                                          {state=normal;break;}
                                       if(c=='\n') ++fc;
                                       break;
                      case inquote:    if(c=='\'') {state=normal;break;}
                                       break;
                      case inparens:   if(c==')') {--parenct; 
                                          if(!parenct) {state=normal; break;}}
                                        /* ignore comments inside parens */
                                       if(c=='*' && lastc=='(' && parenct==1)
                                       {--parenct;  
                                           state=incomment; 
                                           ++fc; 
                                           keycnt=0; 
                                           break;}
                                       if(c=='(') {++parenct; keycnt=0; break;}
                                       if(c=='\'') {state=inqinp; break;}
                                       break;
                     case inqinp:     if(c=='\'') state=inparens;
                                       break;
                        default:         ;
                        }
                llastc=lastc;
                lastc=c;
                }

        printf( "%13s:\t%6ld %6ld %6ld %6ld %6ld \n", fn, fs, fc, ff, fb, fl);
        fprintf( fp_out,"%13s:\t%6ld %6ld %6ld %6ld %6ld\n",fn,fs,fc,ff,fb,fl);

        tc += fc;
        tl += fl;
        ts += fs;
        tb += fb;
        tp += ff;
}
