/*
 *      esm.c
 *
 *      This program plots Bearing and Amplitude reports.
 *
 *      Developed for IBM EGA/VGA Graphic Adaptors using Borland C++ V2.0
 *
 *      Public Domain (p) April 1991, 1992, S. Sampson, N5OWK
 *
 *	There are 256 pseudo-degrees in the circle.  Each azimuth may
 *	contain one target.  This program will process the raw video
 *	using a sliding window algorithm to produce peak detects.  The
 *	peak is displayed as a red X.  The higher the amplitude, the
 *	closer to the center it is displayed.  The unipolar video is
 *	converted to digital using an 8 Bit A/D.  Therefore an amplitude
 *	of 255 will display in the center, and 1 will display at the
 *	outside range.
 */

/* Includes */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <conio.h>
#include <alloc.h>
#include <dos.h>
#include <bios.h>
#include <graphics.h>
#include <mouse.h>

/* Defines */

typedef unsigned long ulong;
typedef unsigned char uchar;
typedef unsigned int uint;

#define EraseTrack(x, y)	PlotTrack((x), (y))

/* not used yet */

#define	DEBUG

#define ESC             27
#define LEFTKEY         331
#define RIGHTKEY        333
#define ENDKEY          335
#define DOWNKEY         336
#define PGDNKEY         337
#define INSKEY          338
#define DELKEY          339
#define CTRLLEFTKEY     371
#define CTRLRIGHTKEY    372
#define F1              315
#define F2              316
#define F3              317
#define F4              318
#define F5              319
#define F6              320
#define F7              321
#define F8              322
#define F9              323
#define F10             324
#define	F11		389
#define	F12		390
#define HOMEKEY         327
#define UPKEY           328
#define PGUPKEY         329

/* Globals */

M_POS   *Mptr;

ulong far *Time;			/* pointer to BIOS time		*/

int     GraphDriver = DETECT;           /* graphics device driver       */
int     GraphMode;                      /* graphics mode value          */
uint    CenterX, CenterY;               /* center of screen             */
uchar   Sw1, Sw2, Sw4, Sw5;             /* Five Bit Sliding window      */

uchar   CanTgtFile[256];                /* Detections each pass         */
uchar   FwdTgts1[256];                  /* First pass detects           */
uchar   FwdTgts2[256];                  /* Second pass detects          */

uchar   *ThisPass = FwdTgts1;
uchar   *LastPass = FwdTgts2;

void    *Image;                         /* Target Symbol Image          */

#ifdef DEBUG
uchar   Pos1 = 72;      		/* Moving test target 1 window  */
uchar   Pos2 = 73;
uchar   Pos3 = 74;
uchar	Pos4 = 75;
uchar   Pos5 = 76;

uchar   PPos1 = 22;      		/* Moving test target 2 window  */
uchar   PPos2 = 23;
uchar   PPos3 = 24;
uchar	PPos4 = 25;
uchar   PPos5 = 26;
#endif

/*
 *      This is a square mouse cursor with a pointer dot in the center
 */

uint Cursor[] = {
        0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xF00F, 0xF7EF, 0xF7EF, 0xF66F,
        0xF7EF, 0xF7EF, 0xF00F, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF,
        0x0000, 0x0000, 0x0000, 0x0000, 0x0FF0, 0x0810, 0x0810, 0x0990,
        0x0810, 0x0810, 0x0FF0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000
};

/*
 *      8-Bit Sine Degree Table for n = 0 to 255
 *
 *	This program was designed to work with EGA Graphics.
 *	VGA is supported only in an EGA compatible mode.
 *	EGA = 0.775 Aspect Ratio
 *
 *      Sine[n] = sin(x) * 127, x = x + (360 / 256)
 *	Get the Sine values into integer 7 bits plus sign form
 */

char	Sine[] = {
	0, 3, 6, 9, 12, 15, 18, 21, 24, 27, 30, 33, 36, 39, 42, 45,
	48, 51, 54, 57, 59, 62, 65, 67,	70, 73, 75, 78, 80, 82, 85, 87,
	89, 91, 94, 96, 98, 100, 102, 103, 105, 107, 108, 110, 112, 113,
	114, 116, 117, 118, 119, 120, 121, 122, 123, 123, 124, 125, 125,
	126, 126, 126, 126, 126, 127, 126, 126, 126, 126, 126, 125, 125,
	124, 123, 123, 122, 121, 120, 119, 118, 117, 116, 114, 113, 112,
	110, 108, 107,	105, 103, 102, 100, 98, 96, 94, 91, 89, 87, 85,
	82, 80, 78, 75, 73, 70, 67, 65, 62, 59, 57, 54, 51, 48, 45, 42,
	39, 36, 33, 30, 27, 24, 21, 18, 15, 12, 9, 6, 3, 0, -3, -6, -9,
	-12, -15, -18, -21, -24, -27, -30, -33, -36, -39, -42, -45, -48,
	-51, -54, -57, -59, -62, -65, -67, -70, -73, -75, -78, -80, -82,
	-85, -87, -89, -91, -94, -96, -98, -100, -102, -103, -105, -107,
	-108, -110, -112, -113, -114, -116, -117, -118, -119, -120, -121,
	-122, -123, -123, -124, -125, -125, -126, -126, -126, -126, -126,
	-127, -126, -126, -126, -126, -126, -125, -125,	-124, -123, -123,
	-122, -121, -120, -119, -118, -117, -116, -114, -113, -112, -110,
	-108, -107, -105, -103, -102, -100, -98, -96, -94, -91, -89, -87,
	-85, -82, -80, -78, -75, -73, -70, -67, -65, -62, -59, -57, -54,
	-51, -48, -45, -42, -39, -36, -33, -30, -27, -24, -21, -18, -15,
	-12, -9, -6, -3
};

/*
 *	8-Bit Cosine Degree Table for n = 0 to 255
 *
 *	Cosine[n] = cos(x) * Aspect * 127, x = x + (360 / 256)
 */

char	Cosine[] = {
	98, 98, 98, 98, 97, 97, 97, 96, 96, 96, 95, 94, 94, 93, 92, 91,
	90, 89, 88, 87, 86, 85, 84, 83, 81, 80, 79, 77, 76, 74, 72, 71,
	69, 67, 66, 64, 62, 60, 58, 56, 54, 52, 50, 48, 46, 44, 42, 39,
	37, 35, 33, 30, 28, 26, 23, 21, 19, 16, 14, 12, 9, 7, 4, 2,
	0, -2, -4, -7, -9, -12, -14, -16, -19, -21, -23, -26, -28, -30,
	-33, -35, -37, -39, -42, -44, -46, -48, -50, -52, -54, -56, -58,
	-60, -62, -64, -66, -67, -69, -71, -72, -74, -76, -77, -79, -80,
	-81, -83, -84, -85, -86, -87, -88, -89, -90, -91, -92, -93, -94,
	-94, -95, -96, -96, -96, -97, -97, -97, -98, -98, -98, -98, -98,
	-98, -98, -97, -97, -97, -96, -96, -96, -95, -94, -94, -93, -92,
	-91, -90, -89, -88, -87, -86, -85, -84, -83, -81, -80, -79, -77,
	-76, -74, -72, -71, -69, -67, -66, -64, -62, -60, -58, -56, -54,
	-52, -50, -48, -46, -44, -42, -39, -37, -35, -33, -30, -28, -26,
	-23, -21, -19, -16, -14, -12, -9, -7, -4, -2, 0, 2, 4, 7, 9, 12,
	14, 16, 19, 21, 23, 26, 28, 30, 33, 35, 37, 39, 42, 44, 46, 48,
	50, 52, 54, 56, 58, 60, 62, 64, 66, 67, 69, 71, 72, 74, 76, 77,
	79, 80, 81, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 94,
	95, 96, 96, 96, 97, 97, 97, 98, 98, 98
};

/* Function Prototypes */

void    InitGraphics(void), GetCommand(void), GenTarget(void);
void    ScanForTgt(void), convert(int, int, int *, int *);
void    PlotTrack(int, int), LoadCandidate(void), readscreen(char *);
void	ex_command(char *), beeper(void);
int     gprintf(int xloc, int yloc, char *fmt, ... );
int     quit(void), getkey(void);

/* Main Program */

void main()
{
        int	i;

	Time = MK_FP(0x40, 0x6c);

        if ((i = m_init()) == 0)  {
                fprintf(stderr, "Fatal - Mouse driver not installed\n");
                exit(1);
        }

        if (i == 2)  {
                fprintf(stderr, "Fatal - You need DOS 2.0 or greater\n");
                exit(1);
        }

        ctrlbrk(quit);
        setcbrk(1);

#ifdef DEBUG
	srand((*Time) % 65535);   /* make a little different on each run */
#endif

        InitGraphics();
        m_graph_cursor(7, 7, (void far *)Cursor);
	m_window(1, 1, 439, 348);
        m_display(ON);

        memset((uchar *)FwdTgts1, 0, 256);
        memset((uchar *)FwdTgts2, 0, 256);

        for (;;)  {
		LoadCandidate();
                ScanForTgt();
                GenTarget();

                if (kbhit())
		        GetCommand();
        }
}

/* Functions */

/*
 *      Get the keyboard command entered
 *	Commands are 20 characters max.
 */

void GetCommand()
{
        int             i, ch;
	static char	ocommand[23], command[23];
	static char	*cp = command;

        setviewport(441, 1, 638, 348, 1);

	ch = getkey();
	if (ch < 224)  {

		ch = toupper(ch);

		switch (ch)  {
		case '\r':
			/* execute command line buffer */

			if (cp != command)  {
				ex_command(command);
			        setcolor(EGA_DARKGRAY);
			        gprintf(20, 330, "%s", command);
			        setcolor(EGA_LIGHTGRAY);
				cp = command;
				*cp = ocommand[0] = '\0';
			}
			break;
		case 8:
			/* ^H delete a character backwards */

		        setcolor(EGA_DARKGRAY);
		        gprintf(20, 330, "%s", ocommand);
			cp--;
			*cp = '\0';
		        setcolor(EGA_LIGHTGRAY);
		        gprintf(20, 330, "%s", command);
			strcpy(ocommand, command);
			break;
		case 21:
			/* ^U delete the whole buffer */

		        setcolor(EGA_DARKGRAY);
		        gprintf(20, 330, "%s", ocommand);
			cp = command;
			*cp = ocommand[0] = '\0';
			break;
		case 0:
			break;
		default:
			/* add character to command line buffer */
	
			if (cp > (command + 20))
				beeper();
			else  {
			        setcolor(EGA_DARKGRAY);
			        gprintf(20, 330, "%s", ocommand);
				*cp++ = ch;
				*cp = '\0';
			        setcolor(EGA_LIGHTGRAY);
			        gprintf(20, 330, "%s", command);
				strcpy(ocommand, command);
		        }
		}
	}

        setviewport(1, 1, 439, 348, 1);
}

void beeper()
{
        sound(800);
        delay(125);
        nosound();
}

int getkey()
{
        uint    lo, hi, key;

        key = bioskey(0);
        lo  = key & 0x00FF;
        hi  = (key & 0xFF00) >> 8;

	return ((lo == 0) ? (hi + 256) : lo);
}

void ex_command(char *command)
{
	if (strcmp(command,"QUIT") == 0)
		quit();
	else if (strcmp(command,"CURSOR ON") == 0)
		m_display(ON);
	else if (strcmp(command,"CURSOR OFF") == 0)
		m_display(OFF);
}

/*
 *      Free all data buffers and return to DOS
 */

#pragma warn -rvl
int quit()
{
        free(Image);
        closegraph();
        m_display(OFF);
	beeper();

        exit(0);
}
#pragma warn +rvl

#ifdef DEBUG
/*
 *      Input test data into the candidate target array
 */

void LoadCandidate()
{
        memset(CanTgtFile, 0, 256);

        CanTgtFile[189] = 1;    /* W Test Target */
        CanTgtFile[190] = 1;
        CanTgtFile[191] = 3;
        CanTgtFile[192] = 1;
        CanTgtFile[193] = 1;

        CanTgtFile[90] = 32;   /* SE Test Target */
        CanTgtFile[91] = 50;
        CanTgtFile[92] = 84;
        CanTgtFile[93] = 50;
        CanTgtFile[94] = 32;

        CanTgtFile[254] = 10;   /* N Test Target */
        CanTgtFile[255] = 10;
        CanTgtFile[000] = 20;
        CanTgtFile[001] = 9;
        CanTgtFile[002] = 9;

	if (random(256) > 128)  {
	        CanTgtFile[PPos1--] = 30;	/*  Moving Test Target 1 */
        	CanTgtFile[PPos2--] = 46;
        	CanTgtFile[PPos3--] = 80;
        	CanTgtFile[PPos4--] = 44;
        	CanTgtFile[PPos5--] = 22;
	} else {
	        CanTgtFile[PPos1++] = 30;
	        CanTgtFile[PPos2++] = 46;
        	CanTgtFile[PPos3++] = 80;
        	CanTgtFile[PPos4++] = 44;
	        CanTgtFile[PPos5++] = 22;
	}

        CanTgtFile[Pos1++] = 2;		/*  Moving Test Target 2 */
        CanTgtFile[Pos2++] = 4;
       	CanTgtFile[Pos3++] = 127;
       	CanTgtFile[Pos4++] = 4;
        CanTgtFile[Pos5++] = 2;
}
#else
void LoadCandidate()
{
}
#endif

/*
 *      Run the Sliding Window (sw) through the data
 *      and produce peak detects
 */

void ScanForTgt()
{
        int    i, sw3;

        Sw1 = 254;      /* Sliding window */
        Sw2 = 255;
        Sw4 = 001;
        Sw5 = 002;

        for (i = 0; i < 256; i++)  {

                /* threshold is twice the average of surrounding signal */

		sw3  = CanTgtFile[Sw1];
		sw3 += CanTgtFile[Sw2];
		sw3 += CanTgtFile[Sw4];
		sw3 += CanTgtFile[Sw5];
		sw3 >>= 2;		/* divide by 4 (truncate) */
		sw3 <<= 1;		/* multiply by 2 */
		sw3 &= 0xFF;		/* normalize to range 0 - 255 */

                if (CanTgtFile[i] >= sw3)
                        ThisPass[i] = (CanTgtFile[i] * CenterX) >> 8;
                else
                        ThisPass[i] = 0;

		Sw1++;
		Sw2++;
		Sw4++;
		Sw5++;
        }
}

void GenTarget()
{
	int	t1, t2, i, x, y;
	uchar	*tmp;

	for (i = 0; i < 256; ++i)  {

		t1 = LastPass[i];
		t2 = ThisPass[i];

		if (t2)  {

			/*
			 *	We have an amplitude on this pass
			 */

			if (t2 == t1)  {

				/*
				 *	Amplitude was already plotted
				 */

				continue;
			} else if (t1) {

				/*
				 *	Different amplitude was plotted
				 *	on last pass, so erase old amplitude
				 */

				convert(i, CenterX-t1, &x, &y);
				EraseTrack(x, y);

				/*
				 *	And plot new amplitude
				 */

				convert(i, CenterX-t2, &x, &y);
				PlotTrack(x, y);
			} else  {

				/*
				 *	No amplitudes plotted on last pass
				 */

				convert(i, CenterX-t2, &x, &y);
				PlotTrack(x, y);
			}
		} else if (t1)  {

			/*
			 *	No amplitude on this pass but one on last
			 */

			convert(i, CenterX-t1, &x, &y);
			EraseTrack(x, y);
		}
	}

	/*
	 *	Swap the data pointers
	 */

	tmp = ThisPass;
	ThisPass = LastPass;
	LastPass = tmp;
}

void PlotTrack(int x, int y)
{
        setviewport(0, 0, 639, 348, 1);

        Mptr = m_status();

	if (Mptr->display == ON)  {
                m_display(OFF);
                putimage(x, y, Image, XOR_PUT);
                m_display(ON);
        } else
                putimage(x, y, Image, XOR_PUT);
}

/*
 *      Initialize the graphics system
 */

void InitGraphics()
{
        struct palettetype pal;
        uint    size;

        registerbgidriver(EGAVGA_driver);
        initgraph(&GraphDriver, &GraphMode, "");

        if (GraphMode == EGAHI)
                ;
        else if (GraphMode == VGAHI)  {
                GraphMode = VGAMED;
                setgraphmode(VGAMED);
        } else  {
                fprintf(stderr, "Fatal - Program requires EGA or VGA.\n");
                exit(1);
        }

        getpalette(&pal);

        /* Fill in global constants */

        CenterX  = 219;			/* EGA center (moved left 100 pix) */
        CenterY  = 174;

        /*
         *      Draw the hello screen, border, and scope
         */

        cleardevice();                  /* Clear graphics screen */
        setactivepage(1);
        setvisualpage(0);

        readscreen("esm.img");          /* Skip it if it's missing */

        setvisualpage(1);		/* draw screen in background */
        delay(10);			/* prime delay for later use */

        getch();			/* operator hits key to move on */

        cleardevice();
        setallpalette(&pal);

	/*
	 *	Create Target Symbol
	 */

	setcolor(EGA_GREEN);
	circle(23, 14, 3);
        setfillstyle(SOLID_FILL, EGA_GREEN);
        floodfill(23, 14, EGA_GREEN);

	setcolor(EGA_LIGHTGREEN);
	rectangle(22, 13, 24, 15);

	size = imagesize(19, 11, 28, 27);
	Image = malloc(size);

	getimage(19, 11, 28, 27, Image);
	putimage(19, 11, Image, XOR_PUT);

        setactivepage(0);

	/* Draw some range rings */

        setcolor(EGA_DARKGRAY);

	circle(CenterX, CenterY, 216);
	circle(CenterX, CenterY, 162);
	circle(CenterX, CenterY, 108);
	circle(CenterX, CenterY,  54);

        setcolor(EGA_LIGHTGRAY);

        rectangle(0, 0, 639, 349);
        line(440, 0, 440, 349);

        setfillstyle(SOLID_FILL, EGA_DARKGRAY);
        floodfill(450, 10, EGA_LIGHTGRAY);

        line(440, 320, 639, 320);

        setviewport(1, 1, 439, 348, 1);
        setvisualpage(0);
}

void convert(int bearing, int amp, int *x, int *y)
{
        /*
         *      Convert polar to rectangular
         */

        *x = CenterX + ((amp * Sine[bearing]) >> 7);
        *y = CenterY - ((amp * Cosine[bearing]) >> 7);
}

int gprintf(int xloc, int yloc, char *fmt, ... )
{
        va_list argptr;                 /* Argument list pointer        */
        char    str[140];               /* Buffer to build string into  */
        int     cnt;                    /* Result of sprintf for return */

        va_start(argptr, fmt);                  /* initialize va_ functions */
        cnt = vsprintf(str, fmt, argptr);       /* prints string to buffer */
        outtextxy(xloc, yloc, str);             /* Send in graphics mode */
        va_end(argptr);                         /* Close va_ functions */

        return cnt;                             /* Return conversion count */
}

void readscreen(char *fn)
{
#pragma option -a	/* Align on word boundary (won't work otherwise) */
        struct {
                char    id[8];
                unsigned size;
                struct palettetype palette;
        } hdr;
#pragma option -a.

        register int    i;
        FILE    *fh;
        void    *buff;

        if ((fh = fopen(fn, "rb")) == NULL)   /* open graphics file */
                return;

        fread(&hdr, sizeof(hdr), 1, fh);/* read size (to malloc) & palette */
        setallpalette(&hdr.palette);    /* set palette to palette from file */

        if ((buff = malloc(hdr.size)) == NULL)  /* alloc memory */
                return;

        for (i = 0; i < 350; i += 70)  {
                fread(buff, hdr.size, 1, fh);   /* read 5 horizonal images */
                putimage(0, i, buff, COPY_PUT); /* put images on the page  */
        }

        free(buff);                                /* clean up */
        fclose(fh);
}
