//=====================================================================
//
//  dpmirun.cpp
//
//  This is the loader for protected mode dos applications.
//
//  This is very dependant on Borland C++.
//  It must be compiled in small model.
//
//  It requires DPMI to work.
//
//  Copyright (c) 1994, Kevin Morgan, All rights reserved.
//
//=====================================================================

#define GLOBALS
#include "dpmirun.h"

DpmiApplication Dpmi;

#define CONTINUE(n)  do {  \
	pGpfRegs  = (GpfRegs far *) MK_FP(_SS, _BP); \
	if (setjmp(int21buf)==0) longjmp(trapbuf,n); } while (0)

#define SWITCHBACK(x)  longjmp(int21buf,x)

#ifndef DEBUG
#define PRINTME(n)
#else
#define PRINTME(n)  CONTINUE(n)
#endif
//=====================================================================
//
//  GpfRegs
//
//  This is what our stack looks like after we have entered our
//  exception handler.
//
//  This is heavily dependant on order that Borland C pushes
//  registers onto the stack.
//
//=====================================================================
struct GpfRegs {
	WORD Rbp;
    WORD Rdi;
    WORD Rsi;
    WORD Rds;
	WORD Res;
    WORD Rdx;
    WORD Rcx;
    WORD Rbx;
	WORD Rax;
    WORD Rip;
    WORD Rcs;
    WORD Rfl;
	WORD faultip;
    WORD faultcs;
    WORD flags;
	WORD sp;
    WORD ss;
};


//=====================================================================
//
//  global variables
//
//=====================================================================
unsigned realDs;        // real mode segment for our DS

unsigned realCs;        // real mode segment for our CS

unsigned protDsAlias;   // CS alias for our DS

jmp_buf trapbuf;        // for a long jump to our exception handler

jmp_buf int21buf;       // for a long jump to coroutine to handle int 21h

GpfRegs gpfRegs;        // copy of registers at the time of the exception

GpfRegs far * pGpfRegs; // points to original registers at int/fault

DpmiInterruptVector oldInt21, oldInt61, oldCtrlC;

ModuleLoader *module;   // represents the .EXE we're trying to run

//=====================================================================
//
//  printDpmiException
//
//  print out our machine state, and attempt to do a traceback
//
//=====================================================================
void printDpmiException(ModuleLoader *module)
{
	int i;
	GpfRegs * pGpfRegs  = &gpfRegs;

	kprintf("\nProcessor Exception:\n");
	kprintf("CS=%04x, DS=%04x  SS:SP=%04x:%04x\n",  _CS, _DS, _SS, _SP);

    kprintf("bp=%04x, di=%04x, si=%04x ds=%04x\n",
        pGpfRegs->Rbp,
        pGpfRegs->Rdi,
        pGpfRegs->Rsi,
        pGpfRegs->Rds);
    kprintf("es=%04x, dx=%04x, cx=%04x bx=%04x\n",
        pGpfRegs->Res,
        pGpfRegs->Rdx,
        pGpfRegs->Rcx,
        pGpfRegs->Rbx);
    kprintf("ax=%04x, ip=%04x, cs=%04x fl=%04x\n",
        pGpfRegs->Rax,
        pGpfRegs->Rip,
        pGpfRegs->Rcs,
        pGpfRegs->Rfl);

    kprintf("Fault CS:IP = %04x:%04x Flags=%04x\n",
        pGpfRegs->faultcs,
        pGpfRegs->faultip,
        pGpfRegs->flags);

    kprintf("ss:sp = %04x:%04x\n",
        pGpfRegs->ss,
        pGpfRegs->sp);

	static jmp_buf savetrap;
	memcpy(savetrap, trapbuf, sizeof(jmp_buf));

	if (setjmp(trapbuf)==0) {   // a fault here is ok
		printf("\nStack traceback follows:\n");
		{
			char *faultDescr = module->describeFaultPc(
                                    pGpfRegs->faultcs,
                                    pGpfRegs->faultcs,
                                    pGpfRegs->faultip);
			kprintf("Fault at  %s\n", faultDescr);
		}
		unsigned bp = pGpfRegs->Rbp;
        unsigned nearCs = pGpfRegs->faultcs;
		for (i=0;i<20;i++) {
			unsigned far *p = (unsigned far *) MK_FP(pGpfRegs->ss, bp);
			char *faultDescr = module->describeFaultPc(
									nearCs, p[2], p[1]);
			kprintf("call from %s\n", faultDescr);
			if (p[0]<=bp) break;
			if (p[0]==0xffff) break;
			bp = p[0];
		}
	}
	memcpy(trapbuf, savetrap, sizeof(jmp_buf));
}


//=====================================================================
//
//  patchExceptionHandler
//
//  Before we can install a protected mode interrupt handler, we
//  need to modify the interrupt procedure's prolog, because of
//  the way BorlandC generates code...
//
//  This is very specific to this implementation
//  and very easy to break!
//
//  In the prolog for Borland C interrupt procedures contains
//  code to push the registers, and load DS with DGROUP.
//
//          ...
//          MOV AX, DGROUP
//          MOV DS, AX
//          ...
// 
//  The problem is that this is compiled in with the
//  DGROUP value of our real mode data segment.
//
//  Since this is a protected mode interrupt handler,
//  we need that to be the protected mode selector
//  for our data segment (not known till run time).
//
//  So we actually modify our code at run time to
//  have the right value.
//
//  To accomplish this, we create a writeable alias selector,
//  modify the code, and get rid of the alias.
//
//=====================================================================
void patchExceptionHandler(DpmiInterruptVector v)
{
	unsigned WriteableCS;
	unsigned far *p = (unsigned far *) v;

	if (Dpmi.createAlias(_CS, WriteableCS)!=DPMI_OK) {
		printf("patchExceptionHandler: cannot create CS alias\n");
		return;
	}

	FP_SEG(p) = WriteableCS;
	p[5] = _DS;
	Dpmi.freeSelector(WriteableCS);
}

//=====================================================================
//
//  installExceptionHandler
//
//  Install an exception handler to handle all protected mode processor
//  exceptions.
//
//=====================================================================
void installExceptionHandler(DpmiInterruptVector v)
{
    memset(&gpfRegs, 0, sizeof(gpfRegs));
	FP_SEG(v) = _CS;
	int i;
    patchExceptionHandler( v );
//	for (i=0;i<32;i++)
//		Dpmi.setExceptionHandler(i, v);

    Dpmi.setExceptionHandler( 0, v);
    Dpmi.setExceptionHandler( 6, v);
    Dpmi.setExceptionHandler( 7, v);
    Dpmi.setExceptionHandler( 9, v);
    Dpmi.setExceptionHandler(12, v);
    Dpmi.setExceptionHandler(13, v);

//	kprintf("exception handler traps set\n");
}

//=====================================================================
//
//  installIntHandler
//
//  Install an exception handler to handle all protected mode processor
//  exceptions.
//
//=====================================================================
DpmiInterruptVector installIntHandler(int intno, DpmiInterruptVector v)
{
    DpmiInterruptVector oldVect;
	FP_SEG(v) = _CS;
    patchExceptionHandler( v );
	oldVect = Dpmi.getProtVect(intno);
    Dpmi.setProtVect(intno, v); 
    return oldVect;
}

//=====================================================================
//
//  copyArgs
//
//  copy the argv and envp arrays. Mainly needed because we need to
//  convert from an array of near pointers to an array of far pointers.
//
//=====================================================================
char far * far * copyArgs(int num, char **argv)
{
    if (num==0) {
        char **p = argv;
        while (*p++) num++;
    }
    char far * far * table = new char far * [num+1];
    int i;
    for (i=0;i<num;i++)
        table[i] = argv[i];
    table[num] = 0;
    return table;
}

//=====================================================================
//
//  ctrl_c
//
//  This is a routine to catch Control-C interrupts
//  
//
//=====================================================================
void far interrupt ctrl_c(...)
{
	pGpfRegs  = (GpfRegs far *) MK_FP(_SS, _BP);
	longjmp(trapbuf,2);
}

//=====================================================================
//
//  ReflectedRegs
//
//  an instance for each interrupt we reflect from real mode
//  to protected mode.
//
//=====================================================================
class ReflectedRegs {
    public:
        DPMI_Regs dregs;

        DpmiInterruptVector func;

        struct {            // this is actually a bit of code to
            char opCall;    // call our protected mode reflector
            unsigned off;
            unsigned seg;
            char iRet;
        } isr;
        unsigned long counter;
        DpmiInterruptVector oldVect;
        DpmiInterruptVector oldPVect;
        int intNr;
        unsigned magic;
};


//=====================================================================
//
//  reflectToProtMode
//
//  reflect a real mode interrupt to our protected mode handler
//  (gets called in protected mode mode)
//=====================================================================
#pragma argsused
void far interrupt reflectToProtMode(unsigned bp, unsigned di,
    unsigned si, unsigned ds, unsigned es)
{
    ReflectedRegs far *caller = 
            (ReflectedRegs far *) MK_FP(es, di);
    unsigned far * stack = 
            (unsigned far *) MK_FP(ds, si);
    (*caller->func)();  // reflect the interrupt in protected mode
    caller->counter++;
    caller->dregs.cs = stack[1];
    caller->dregs.ip = stack[0];
    caller->dregs.sp += 4;
}

//=====================================================================
//
//  installIntHandler
//
//  install reflecting real mode interrupt handler
//
//=====================================================================
int installIntHandler(GpfRegs far *pGpfRegs)
{
	int intNr = pGpfRegs->Rbx;
	DpmiInterruptVector func;
	ReflectedRegs *iregp;
	func = ( DpmiInterruptVector ) MK_FP(pGpfRegs->Rcx, pGpfRegs->Rdx);
	iregp = (ReflectedRegs *) new ReflectedRegs;

	DpmiInterruptVector callback;

	{
		DpmiInterruptVector h = (DpmiInterruptVector) reflectToProtMode;
		patchExceptionHandler(h);
		FP_SEG(h) = _CS;
		if (Dpmi.allocateRealCallback(h, &iregp->dregs, callback)!=DPMI_OK) {
			return DPMI_ERR;
		}
	}
	iregp->magic      = 0x7079;
	iregp->isr.opCall = 0x9a;   // call instruction
	iregp->isr.seg    = FP_SEG(callback);
	iregp->isr.off    = FP_OFF(callback);
	iregp->isr.iRet   = 0xcf;   // iret
	iregp->func       = func;
	iregp->counter    = 0;
	iregp->intNr      = intNr;
	iregp->oldVect    = Dpmi.getRealVect(intNr);
	iregp->oldPVect   = Dpmi.getProtVect(intNr);

	unsigned char far *pIsr = (unsigned char far *) &iregp->isr;   // compiler fills in a protected mode selector
	FP_SEG(pIsr) = realDs;

	Dpmi.setRealVect(intNr, (DpmiInterruptVector) pIsr);
	Dpmi.setProtVect(intNr, func);
	pGpfRegs->Rbx = (unsigned) iregp;

	return DPMI_OK;
}

//=====================================================================
//
//  removeIntHandler
//
//  remove real mode interrupt handler
//
//=====================================================================
void removeIntHandler(GpfRegs far *pGpfRegs)
{
	ReflectedRegs *iregp = (ReflectedRegs *) pGpfRegs->Rbx;
	if (iregp->magic  != 0x7079) {
		pGpfRegs->Rax = 1;
		return;
	}
	Dpmi.setRealVect(iregp->intNr, iregp->oldVect);
	Dpmi.setProtVect(iregp->intNr, iregp->oldPVect);
	iregp->magic = 0;
	delete iregp;
	pGpfRegs->Rax = 0;
}

//=====================================================================
//
//  processInt61
//
//  coroutine to process int61 calls
//
//=====================================================================
void processInt61(ModuleLoader *)
{
    // this is where we put processing for any calls that have to
    // run in our stack space.
    // we swap stacks by doing a setjmp/longjmp which takes us to here.
    // When we're done, we switch back by doing a mirror setjmp/longjmp
    // Of course this means that this code is not re-entrant, so it's
    // best to avoid it altogether.
    //
	switch (pGpfRegs->Rax) {
		case EXAPI_INSTALL:  // install reflector
			{
				installIntHandler(pGpfRegs);
			}
			break;

		case EXAPI_REMOVE:
			{
				removeIntHandler(pGpfRegs);
			}
			break;
		case EXAPI_INITCOMM:
            {
                initComm(pGpfRegs->Rbx, pGpfRegs->Rcx, pGpfRegs->Rdx);
                pGpfRegs->Rax = 0;
            }
            break;
        case EXAPI_TERMCOMM:
            {
                termComm(pGpfRegs->Rbx);
                pGpfRegs->Rax = 0;
			}
            break;
        default:
            {
                char *faultDescr = module->describeFaultPc(
                                        pGpfRegs->Rcs,
                                        pGpfRegs->Rcs, pGpfRegs->Rip);
                printf("dosX interrupt at  %s\n", faultDescr);
                fflush(stdout);
            }
    }
    SWITCHBACK(1);
}

//=====================================================================
//
//  int61
//
//  This is a routine to catch int61 calls
//  (my internal debug stuff)
//
//  Be careful here, since _SS != _DS since this is the interface to
//  the user program
//=====================================================================
void far interrupt int61(...)
{
	pGpfRegs  = (GpfRegs far *) MK_FP(_SS, _BP);
	switch (pGpfRegs->Rax) {
        case EXAPI_READCOMM:
            {
                CommRequest far * req = (CommRequest far *)
                        MK_FP(pGpfRegs->Rcx, pGpfRegs->Rdx);
                readComm(pGpfRegs->Rbx, req);
                pGpfRegs->Rax = 0;
            }
            break;
        case EXAPI_WRITECOMM:
            {
                CommRequest far * req = (CommRequest far *)
                        MK_FP(pGpfRegs->Rcx, pGpfRegs->Rdx);
                writeComm(pGpfRegs->Rbx, req);
                pGpfRegs->Rax = 0;
            }
            break;
        default:
            CONTINUE(5);
    }
}

//=====================================================================
//
//  dpmiException
//
//  This is a routine to catch protected mode processor exceptions.
//  
//
//=====================================================================
void far interrupt dpmiException(...)
{
	GpfRegs far * pGpfRegs  = (GpfRegs far *) MK_FP(_SS, _BP);
    _fmemcpy( &gpfRegs, pGpfRegs, sizeof(gpfRegs) );
	longjmp(trapbuf,1);
}

//=====================================================================
//
//  callMain
//
//  Start executing the program we just loaded.
//
//  We allocate a protected mode stack for the new program,
//  copy argument and environment variables, switch to the
//  new stack, and call the program's entrypoint passing
//  argc, argv, and envp.
//
//=====================================================================
void callMain(ModuleLoader *module, void far *startCsIp, int argc, char **argv, char **envp)
{
    const int StackSize = 4096;
    static char far *stk = (char far *) module->getStack(StackSize);
    if (!stk) {
	    Dpmi.fail("cannot allocate protected mode stack\n");
        return;
    }

    stk += (StackSize-32);

	static int (_far *mainline)(int, char far * far *, char far * far *);
	mainline = ( int (_far *)(int, char far * far *, char far * far *) ) startCsIp;


    // strip off the loader's name
    argc--;
    argv++;

    static char far* far* newargv = copyArgs(argc, argv);

    static char far* far* newenvp = copyArgs(0, envp);

    static int newargc = argc;

    switch (setjmp(trapbuf)) {
        case 0:
            // switch stacks 
            flushall();
            asm {
                mov ax, word ptr stk
                mov bx, word ptr stk+2
                mov ss,bx
                mov sp,ax
                push ds
            }

	        (*mainline)(newargc,newargv,newenvp);

	        asm {
                pop ds
            }
            longjmp(trapbuf,3);
            break;
        case 1:
            if (setjmp(trapbuf)==0) {
    		    printDpmiException(module);
            }
		    Dpmi.fail("Processor Exception encountered\n");
            break;
        case 2:
            {
                Dpmi.fail("Control-C intercepted\n");
            }
            break;
        case 3:
            break;
        case 5:
            processInt61(module);
            break;
        default:
            ;
    }
}


//=====================================================================
//
//  protectedMain
//
//  This is the protected mode mainline.
//  We set up exception handler, load the program and run it.
//=====================================================================
int protectedMain(int argc, char *argv[], char *envp[])
{
	if (argc<2) {
		printf("usage: dpmirun filename ...\n");
        Dpmi.dosExit(1);
    }

    module = newModuleLoader();

    static char *where;

    where = "before execution\n";
    if (setjmp(trapbuf)!=0) {
        puts(where);
		Dpmi.fail("Exception encountered before execution\n");
    }

    where = "setup exception handler";
    installExceptionHandler(dpmiException);

    Dpmi.createAlias(_DS, protDsAlias); // for making thunks!

#ifdef DEBUG21
    oldInt21 = installIntHandler(0x21, int21);
#endif

    oldCtrlC = installIntHandler(0x23, ctrl_c);
    oldInt61 = installIntHandler(EXTENDER_VECT, int61);

    where = "error loading executable";
    module->loadExecutable(argv[1]);

    where = "error adjusting selectors";
    module->adjustSelectors();

    where = "error getting entry point";
    void _far *entryPoint = module->entryPoint();

    where = "error check loadErrors";
    if (module->loadErrors()) {
		printf("cannot execute because of load errors\n");
        return 0;
    }

    where = "reset traps";
    callMain(module, entryPoint, argc, argv, envp);

    Dpmi.setProtVect(0x23, oldCtrlC);

    Dpmi.setProtVect(EXTENDER_VECT, oldInt61);

#ifdef DEBUG21
    Dpmi.setProtVect(0x21, oldInt21);
#endif

	return 0;
}


//=====================================================================
//
//  main
//
//  We switch to protected mode, and call the protected mode mainline
//
//=====================================================================
int main(int argc, char *argv[], char *envp[])
{
    int ret;
    
    flushall();     // flush all buffers before switch into protected mode, 
                    // so I/O redirection works properly
    realDs = _DS;
    realCs = _CS;

    if (!Dpmi.present())
      if (!Dpmi.init())
        Dpmi.fail("This program requires DPMI");

    // now in protected mode: segment registers have changed


    ret = protectedMain(argc, argv, envp);

    flushall();

    Dpmi.dosExit(ret);

    return 0;
}
